@@ -1,6 +1,7 @@
 import copy
 from lark import Transformer, v_args, Token
 from optimade.server.mappers import BaseResourceMapper
+from optimade.server.exceptions import BadRequest
 
 
 class MongoTransformer(Transformer):
@@ -24,7 +25,7 @@ class MongoTransformer(Transformer):
     }
 
     def __init__(self, mapper: BaseResourceMapper = None):
-        """ Initialise the object, optionally loading in a
+        """Initialise the object, optionally loading in a
         resource mapper for use when post-processing.
 
         """
@@ -40,6 +41,8 @@ class MongoTransformer(Transformer):
 
         query = self._apply_relationship_filtering(query)
         query = self._apply_length_operators(query)
+        query = self._apply_unknown_or_null_filter(query)
+        query = self._apply_mongo_id_filter(query)
 
         return query
 
@@ -70,7 +73,7 @@ class MongoTransformer(Transformer):
 
     @v_args(inline=True)
     def not_implemented_string(self, value):
-        """ not_implemented_string: value
+        """not_implemented_string: value
 
         Raise NotImplementedError.
         For further information, see Materials-Consortia/OPTIMADE issue 157:
@@ -134,7 +137,10 @@ class MongoTransformer(Transformer):
 
     def known_op_rhs(self, arg):
         # known_op_rhs: IS ( KNOWN | UNKNOWN )
-        return {"$exists": arg[1] == "KNOWN"}
+        # The OPTIMADE spec also required a type comparison with null, this must be post-processed
+        # so here we use a special key "#known" which will get replaced in post-processing with the
+        # expanded dict
+        return {"#known": arg[1] == "KNOWN"}
 
     def fuzzy_string_op_rhs(self, arg):
         # fuzzy_string_op_rhs: CONTAINS value | STARTS [ WITH ] value | ENDS [ WITH ] value
@@ -227,8 +233,8 @@ class MongoTransformer(Transformer):
         )
 
     def _recursive_expression_phrase(self, arg):
-        """ Helper function for parsing `expression_phrase`. Recursively sorts out
-        the correct precedence for $and, $or and $nor.
+        """Helper function for parsing `expression_phrase`. Recursively sorts out
+        the correct precedence for `$and`, `$or` and `$nor`.
 
         """
         if len(arg) == 1:
@@ -254,7 +260,7 @@ class MongoTransformer(Transformer):
         return {prop: {"$not": expr} for prop, expr in arg[1].items()}
 
     def _apply_length_aliases(self, filter_: dict) -> dict:
-        """ Recursively search query for any $size calls, and check
+        """Recursively search query for any `$size` operations, and check
         if the property can be replaced with its corresponding length
         alias.
 
@@ -279,7 +285,7 @@ class MongoTransformer(Transformer):
         )
 
     def _apply_aliases(self, filter_: dict) -> dict:
-        """ Check whether any fields in the filter have aliases so
+        """Check whether any fields in the filter have aliases so
         that they can be renamed for the Mongo query.
 
         """
@@ -303,7 +309,7 @@ class MongoTransformer(Transformer):
         return recursive_postprocessing(filter_, check_for_alias, apply_alias)
 
     def _apply_length_operators(self, filter_: dict) -> dict:
-        """ Check for any invalid pymongo queries that involve
+        """Check for any invalid pymongo queries that involve
         applying an operator to the length of a field, and transform
         them into a test for existence of the relevant entry, e.g.
         "list LENGTH > 3" becomes "does the 4th list entry exist?".
@@ -344,11 +350,13 @@ class MongoTransformer(Transformer):
             return subdict
 
         return recursive_postprocessing(
-            filter_, check_for_length_op_filter, apply_length_op,
+            filter_,
+            check_for_length_op_filter,
+            apply_length_op,
         )
 
     def _apply_relationship_filtering(self, filter_: dict) -> dict:
-        """ Check query for property names that match the entry
+        """Check query for property names that match the entry
         types, and transform them as relationship filters rather than
         property filters.
 
@@ -389,9 +397,88 @@ class MongoTransformer(Transformer):
             filter_, check_for_entry_type, replace_with_relationship
         )
 
+    def _apply_unknown_or_null_filter(self, filter_: dict) -> dict:
+        """This method loops through the query and replaces the check for
+        KNOWN with a check for existence and a check for not null, and the
+        inverse for UNKNOWN.
+
+        """
+
+        def check_for_known_filter(_, expr):
+            """Find cases where the query dict looks like
+            `{"field": {"#known": T/F}}` or
+            `{"field": "$not": {"#known": T/F}}`, which is a magic word
+            for KNOWN/UNKNOWN filters in this transformer.
+
+            """
+            return isinstance(expr, dict) and (
+                "#known" in expr or "#known" in expr.get("$not", {})
+            )
+
+        def replace_known_filter_with_or(subdict, prop, expr):
+            """Replace magic key `"#known"` (added by this transformer) with the appropriate
+            combination of `$exists` and/or test for nullity.
+            combination of $exists and/or $eq/$ne null.
+
+            """
+            not_ = set(expr.keys()) == {"$not"}
+            if not_:
+                expr = expr["$not"]
+
+            exists = expr["#known"] ^ not_
+
+            top_level_key = "$or"
+            comparison_operator = "$eq"
+            if exists:
+                top_level_key = "$and"
+                comparison_operator = "$ne"
+
+            if top_level_key not in subdict:
+                subdict[top_level_key] = []
+
+            subdict[top_level_key].append({prop: {"$exists": exists}})
+            subdict[top_level_key].append({prop: {comparison_operator: None}})
+
+            subdict.pop(prop)
+
+            return subdict
+
+        return recursive_postprocessing(
+            filter_, check_for_known_filter, replace_known_filter_with_or
+        )
+
+    def _apply_mongo_id_filter(self, filter_: dict) -> dict:
+        """This method loops through the query and replaces any operations
+        on the special Mongodb `_id` key with the corresponding operation
+        on a BSON `ObjectId` type.
+        """
+
+        def check_for_id_key(prop, _):
+            """ Find cases where the query dict is operating on the `_id` field. """
+            return prop == "_id"
+
+        def replace_str_id_with_objectid(subdict, prop, expr):
+            from bson import ObjectId
+
+            for operator in subdict[prop]:
+                val = subdict[prop][operator]
+                if operator not in ("$eq", "$ne"):
+                    if self.mapper is not None:
+                        prop = self.mapper.alias_of(prop)
+                    raise BadRequest(
+                        detail=f"Operator not supported for query on field {prop!r}, can only test for equality"
+                    )
+                if isinstance(val, str):
+                    subdict[prop][operator] = ObjectId(val)
+            return subdict
+
+        return recursive_postprocessing(
+            filter_, check_for_id_key, replace_str_id_with_objectid
+        )
+
 
 def recursive_postprocessing(filter_, condition, replacement):
-    """ Recursively descend into the query, checking each dictionary
+    """Recursively descend into the query, checking each dictionary
     (contained in a list, or as an entry in another dictionary) for
     the condition passed. If the condition is true, apply the
     replacement to the dictionary.
