@@ -1,13 +1,21 @@
 # pylint: disable=no-self-argument,line-too-long,no-name-in-module
+import re
 from enum import IntEnum, Enum
 from sys import float_info
-from typing import List, Optional, Tuple, Union
+from typing import List, Optional, Union
 
-from pydantic import Field, BaseModel, validator, root_validator
-
-from .entries import EntryResourceAttributes, EntryResource
-from .utils import CHEMICAL_SYMBOLS, EXTRA_SYMBOLS
+from pydantic import BaseModel, validator, root_validator, conlist
 
+from optimade.models.entries import EntryResourceAttributes, EntryResource
+from optimade.models.utils import (
+    CHEMICAL_SYMBOLS,
+    EXTRA_SYMBOLS,
+    OptimadeField,
+    StrictField,
+    SupportLevel,
+    ANONYMOUS_ELEMENTS,
+    CHEMICAL_FORMULA_REGEXP,
+)
 
 EXTENDED_CHEMICAL_SYMBOLS = CHEMICAL_SYMBOLS + EXTRA_SYMBOLS
 
@@ -26,8 +34,8 @@ __all__ = (
 EPS = float_info.epsilon
 
 
-Vector3D = Tuple[float, float, float]
-Vector3D_unknown = Tuple[Union[float, None], Union[float, None], Union[float, None]]
+Vector3D = conlist(float, min_items=3, max_items=3)
+Vector3D_unknown = conlist(Union[float, None], min_items=3, max_items=3)
 
 
 class Periodicity(IntEnum):
@@ -46,29 +54,32 @@ class StructureFeatures(Enum):
 
 class Species(BaseModel):
     """A list describing the species of the sites of this structure.
-Species can represent pure chemical elements, virtual-crystal atoms representing a
-statistical occupation of a given site by multiple chemical elements, and/or a
-location to which there are attached atoms, i.e., atoms whose precise location are
-unknown beyond that they are attached to that position (frequently used to indicate
-hydrogen atoms attached to another element, e.g., a carbon with three attached
-hydrogens might represent a methyl group, -CH3).
 
-- **Examples**:
-    - `[ {"name": "Ti", "chemical_symbols": ["Ti"], "concentration": [1.0]} ]`: any site with this species is occupied by a Ti atom.
-    - `[ {"name": "Ti", "chemical_symbols": ["Ti", "vacancy"], "concentration": [0.9, 0.1]} ]`: any site with this species is occupied by a Ti atom with 90 % probability, and has a vacancy with 10 % probability.
-    - `[ {"name": "BaCa", "chemical_symbols": ["vacancy", "Ba", "Ca"], "concentration": [0.05, 0.45, 0.5], "mass": 88.5} ]`: any site with this species is occupied by a Ba atom with 45 % probability, a Ca atom with 50 % probability, and by a vacancy with 5 % probability. The mass of this site is (on average) 88.5 a.m.u.
-    - `[ {"name": "C12", "chemical_symbols": ["C"], "concentration": [1.0], "mass": 12.0} ]`: any site with this species is occupied by a carbon isotope with mass 12.
-    - `[ {"name": "C13", "chemical_symbols": ["C"], "concentration": [1.0], "mass": 13.0} ]`: any site with this species is occupied by a carbon isotope with mass 13.
-    - `[ {"name": "CH3", "chemical_symbols": ["C"], "concentration": [1.0], "attached": ["H"], "nattached": [3]} ]`: any site with this species is occupied by a methyl group, -CH3, which is represented without specifying precise positions of the hydrogen atoms.
+    Species can represent pure chemical elements, virtual-crystal atoms representing a
+    statistical occupation of a given site by multiple chemical elements, and/or a
+    location to which there are attached atoms, i.e., atoms whose precise location are
+    unknown beyond that they are attached to that position (frequently used to indicate
+    hydrogen atoms attached to another element, e.g., a carbon with three attached
+    hydrogens might represent a methyl group, -CH3).
+
+    - **Examples**:
+        - `[ {"name": "Ti", "chemical_symbols": ["Ti"], "concentration": [1.0]} ]`: any site with this species is occupied by a Ti atom.
+        - `[ {"name": "Ti", "chemical_symbols": ["Ti", "vacancy"], "concentration": [0.9, 0.1]} ]`: any site with this species is occupied by a Ti atom with 90 % probability, and has a vacancy with 10 % probability.
+        - `[ {"name": "BaCa", "chemical_symbols": ["vacancy", "Ba", "Ca"], "concentration": [0.05, 0.45, 0.5], "mass": 88.5} ]`: any site with this species is occupied by a Ba atom with 45 % probability, a Ca atom with 50 % probability, and by a vacancy with 5 % probability. The mass of this site is (on average) 88.5 a.m.u.
+        - `[ {"name": "C12", "chemical_symbols": ["C"], "concentration": [1.0], "mass": 12.0} ]`: any site with this species is occupied by a carbon isotope with mass 12.
+        - `[ {"name": "C13", "chemical_symbols": ["C"], "concentration": [1.0], "mass": 13.0} ]`: any site with this species is occupied by a carbon isotope with mass 13.
+        - `[ {"name": "CH3", "chemical_symbols": ["C"], "concentration": [1.0], "attached": ["H"], "nattached": [3]} ]`: any site with this species is occupied by a methyl group, -CH3, which is represented without specifying precise positions of the hydrogen atoms.
 
     """
 
-    name: str = Field(
+    name: str = OptimadeField(
         ...,
-        decsription="""Gives the name of the species; the **name** value MUST be unique in the `species` list.""",
+        description="""Gives the name of the species; the **name** value MUST be unique in the `species` list.""",
+        support=SupportLevel.MUST,
+        queryable=SupportLevel.OPTIONAL,
     )
 
-    chemical_symbols: List[str] = Field(
+    chemical_symbols: List[str] = OptimadeField(
         ...,
         description="""MUST be a list of strings of all chemical elements composing this species. Each item of the list MUST be one of the following:
 
@@ -77,9 +88,11 @@ hydrogens might represent a methyl group, -CH3).
 - the special value `"vacancy"` to represent that this site has a non-zero probability of having a vacancy (the respective probability is indicated in the `concentration` list, see below).
 
 If any one entry in the `species` list has a `chemical_symbols` list that is longer than 1 element, the correct flag MUST be set in the list `structure_features`.""",
+        support=SupportLevel.MUST,
+        queryable=SupportLevel.OPTIONAL,
     )
 
-    concentration: List[float] = Field(
+    concentration: List[float] = OptimadeField(
         ...,
         description="""MUST be a list of floats, with same length as `chemical_symbols`. The numbers represent the relative concentration of the corresponding chemical symbol in this species. The numbers SHOULD sum to one. Cases in which the numbers do not sum to one typically fall only in the following two categories:
 
@@ -87,29 +100,39 @@ If any one entry in the `species` list has a `chemical_symbols` list that is lon
 - Experimental errors in the data present in the database. In this case, it is the responsibility of the client to decide how to process the data.
 
 Note that concentrations are uncorrelated between different site (even of the same species).""",
+        support=SupportLevel.MUST,
+        queryable=SupportLevel.OPTIONAL,
     )
 
-    mass: Optional[float] = Field(
+    mass: Optional[float] = OptimadeField(
         None,
         description="""If present MUST be a float expressed in a.m.u.""",
         unit="a.m.u.",
+        support=SupportLevel.OPTIONAL,
+        queryable=SupportLevel.OPTIONAL,
     )
 
-    original_name: Optional[str] = Field(
+    original_name: Optional[str] = OptimadeField(
         None,
         description="""Can be any valid Unicode string, and SHOULD contain (if specified) the name of the species that is used internally in the source database.
 
 Note: With regards to "source database", we refer to the immediate source being queried via the OPTIMADE API implementation.""",
+        support=SupportLevel.OPTIONAL,
+        queryable=SupportLevel.OPTIONAL,
     )
 
-    attached: Optional[List[str]] = Field(
+    attached: Optional[List[str]] = OptimadeField(
         None,
         description="""If provided MUST be a list of length 1 or more of strings of chemical symbols for the elements attached to this site, or "X" for a non-chemical element.""",
+        support=SupportLevel.OPTIONAL,
+        queryable=SupportLevel.OPTIONAL,
     )
 
-    nattached: Optional[List[int]] = Field(
+    nattached: Optional[List[int]] = OptimadeField(
         None,
         description="""If provided MUST be a list of length 1 or more of integers indicating the number of attached atoms of the kind specified in the value of the :field:`attached` key.""",
+        support=SupportLevel.OPTIONAL,
+        queryable=SupportLevel.OPTIONAL,
     )
 
     @validator("chemical_symbols", each_item=True)
@@ -120,11 +143,17 @@ Note: With regards to "source database", we refer to the immediate source being
 
     @validator("concentration")
     def validate_concentration(cls, v, values):
-        if len(v) != len(values.get("chemical_symbols", [])):
-            raise ValueError(
-                f"Length of concentration ({len(v)}) MUST equal length of chemical_symbols ({len(values.get('chemical_symbols', 'Not specified'))})"
-            )
-        return v
+        if "chemical_symbols" in values:
+            if len(v) != len(values["chemical_symbols"]):
+                raise ValueError(
+                    f"Length of concentration ({len(v)}) MUST equal length of chemical_symbols "
+                    f"({len(values.get('chemical_symbols', []))})"
+                )
+            return v
+
+        raise ValueError(
+            "Could not validate concentration as 'chemical_symbols' is missing/invalid."
+        )
 
     @validator("attached", "nattached")
     def validate_minimum_list_length(cls, v):
@@ -162,30 +191,34 @@ Note: With regards to "source database", we refer to the immediate source being
 class Assembly(BaseModel):
     """A description of groups of sites that are statistically correlated.
 
-- **Examples** (for each entry of the assemblies list):
-    - `{"sites_in_groups": [[0], [1]], "group_probabilities: [0.3, 0.7]}`: the first site and the second site never occur at the same time in the unit cell.
-      Statistically, 30 % of the times the first site is present, while 70 % of the times the second site is present.
-    - `{"sites_in_groups": [[1,2], [3]], "group_probabilities: [0.3, 0.7]}`: the second and third site are either present together or not present; they form the first group of atoms for this assembly.
-      The second group is formed by the fourth site. Sites of the first group (the second and the third) are never present at the same time as the fourth site.
-      30 % of times sites 1 and 2 are present (and site 3 is absent); 70 % of times site 3 is present (and sites 1 and 2 are absent).
+    - **Examples** (for each entry of the assemblies list):
+        - `{"sites_in_groups": [[0], [1]], "group_probabilities: [0.3, 0.7]}`: the first site and the second site never occur at the same time in the unit cell.
+          Statistically, 30 % of the times the first site is present, while 70 % of the times the second site is present.
+        - `{"sites_in_groups": [[1,2], [3]], "group_probabilities: [0.3, 0.7]}`: the second and third site are either present together or not present; they form the first group of atoms for this assembly.
+          The second group is formed by the fourth site. Sites of the first group (the second and the third) are never present at the same time as the fourth site.
+          30 % of times sites 1 and 2 are present (and site 3 is absent); 70 % of times site 3 is present (and sites 1 and 2 are absent).
 
     """
 
-    sites_in_groups: List[List[int]] = Field(
+    sites_in_groups: List[List[int]] = OptimadeField(
         ...,
         description="""Index of the sites (0-based) that belong to each group for each assembly.
 
 - **Examples**:
     - `[[1], [2]]`: two groups, one with the second site, one with the third.
     - `[[1,2], [3]]`: one group with the second and third site, one with the fourth.""",
+        support=SupportLevel.MUST,
+        queryable=SupportLevel.OPTIONAL,
     )
 
-    group_probabilities: List[float] = Field(
+    group_probabilities: List[float] = OptimadeField(
         ...,
         description="""Statistical probability of each group. It MUST have the same length as `sites_in_groups`.
 It SHOULD sum to one.
 See below for examples of how to specify the probability of the occurrence of a vacancy.
 The possible reasons for the values not to sum to one are the same as already specified above for the `concentration` of each `species`.""",
+        support=SupportLevel.MUST,
+        queryable=SupportLevel.OPTIONAL,
     )
 
     @validator("sites_in_groups")
@@ -203,7 +236,8 @@ The possible reasons for the values not to sum to one are the same as already sp
     def check_self_consistency(cls, v, values):
         if len(v) != len(values.get("sites_in_groups", [])):
             raise ValueError(
-                f"sites_in_groups and group_probabilities MUST be of same length, but are {len(values.get('sites_in_groups', 'Not specified'))} and {len(v)}, respectively"
+                f"sites_in_groups and group_probabilities MUST be of same length, "
+                f"but are {len(values.get('sites_in_groups', []))} and {len(v)}, respectively"
             )
         return v
 
@@ -211,7 +245,7 @@ The possible reasons for the values not to sum to one are the same as already sp
 class StructureResourceAttributes(EntryResourceAttributes):
     """This class contains the Field for the attributes used to represent a structure, e.g. unit cell, atoms, positions."""
 
-    elements: List[str] = Field(
+    elements: List[str] = OptimadeField(
         ...,
         description="""Names of the different elements present in the structure.
 
@@ -231,9 +265,11 @@ class StructureResourceAttributes(EntryResourceAttributes):
 - **Query examples**:
     - A filter that matches all records of structures that contain Si, Al **and** O, and possibly other elements: `elements HAS ALL "Si", "Al", "O"`.
     - To match structures with exactly these three elements, use `elements HAS ALL "Si", "Al", "O" AND elements LENGTH 3`.""",
+        support=SupportLevel.SHOULD,
+        queryable=SupportLevel.MUST,
     )
 
-    nelements: int = Field(
+    nelements: int = OptimadeField(
         ...,
         description="""Number of different elements in the structure as an integer.
 
@@ -250,9 +286,11 @@ class StructureResourceAttributes(EntryResourceAttributes):
     - Note: queries on this property can equivalently be formulated using `elements LENGTH`.
     - A filter that matches structures that have exactly 4 elements: `nelements=4`.
     - A filter that matches structures that have between 2 and 7 elements: `nelements>=2 AND nelements<=7`.""",
+        support=SupportLevel.SHOULD,
+        queryable=SupportLevel.MUST,
     )
 
-    elements_ratios: List[float] = Field(
+    elements_ratios: List[float] = OptimadeField(
         ...,
         description="""Relative proportions of different elements in the structure.
 
@@ -272,9 +310,11 @@ class StructureResourceAttributes(EntryResourceAttributes):
     - Note: Useful filters can be formulated using the set operator syntax for correlated values.
       However, since the values are floating point values, the use of equality comparisons is generally inadvisable.
     - OPTIONAL: a filter that matches structures where approximately 1/3 of the atoms in the structure are the element Al is: `elements:elements_ratios HAS ALL "Al":>0.3333, "Al":<0.3334`.""",
+        support=SupportLevel.SHOULD,
+        queryable=SupportLevel.MUST,
     )
 
-    chemical_formula_descriptive: str = Field(
+    chemical_formula_descriptive: str = OptimadeField(
         ...,
         description="""The chemical formula for a structure as a string in a form chosen by the API implementation.
 
@@ -298,9 +338,11 @@ class StructureResourceAttributes(EntryResourceAttributes):
     - Note: the free-form nature of this property is likely to make queries on it across different databases inconsistent.
     - A filter that matches an exactly given formula: `chemical_formula_descriptive="(H2O)2 Na"`.
     - A filter that does a partial match: `chemical_formula_descriptive CONTAINS "H2O"`.""",
+        support=SupportLevel.SHOULD,
+        queryable=SupportLevel.MUST,
     )
 
-    chemical_formula_reduced: str = Field(
+    chemical_formula_reduced: str = OptimadeField(
         ...,
         description="""The reduced chemical formula for a structure as a string with element symbols and integer chemical proportion numbers.
 The proportion number MUST be omitted if it is 1.
@@ -325,9 +367,12 @@ The proportion number MUST be omitted if it is 1.
 
 - **Query examples**:
     - A filter that matches an exactly given formula is `chemical_formula_reduced="H2NaO"`.""",
+        support=SupportLevel.SHOULD,
+        queryable=SupportLevel.MUST,
+        regex=CHEMICAL_FORMULA_REGEXP,
     )
 
-    chemical_formula_hill: Optional[str] = Field(
+    chemical_formula_hill: Optional[str] = OptimadeField(
         None,
         description="""The chemical formula for a structure in [Hill form](https://dx.doi.org/10.1021/ja02046a005) with element symbols followed by integer chemical proportion numbers. The proportion number MUST be omitted if it is 1.
 
@@ -353,9 +398,12 @@ The proportion number MUST be omitted if it is 1.
 
 - **Query examples**:
     - A filter that matches an exactly given formula is `chemical_formula_hill="H2O2"`.""",
+        support=SupportLevel.OPTIONAL,
+        queryable=SupportLevel.OPTIONAL,
+        regex=CHEMICAL_FORMULA_REGEXP,
     )
 
-    chemical_formula_anonymous: str = Field(
+    chemical_formula_anonymous: str = OptimadeField(
         ...,
         description="""The anonymous formula is the `chemical_formula_reduced`, but where the elements are instead first ordered by their chemical proportion number, and then, in order left to right, replaced by anonymous symbols A, B, C, ..., Z, Aa, Ba, ..., Za, Ab, Bb, ... and so on.
 
@@ -372,9 +420,12 @@ The proportion number MUST be omitted if it is 1.
 
 - **Querying**:
     - A filter that matches an exactly given formula is `chemical_formula_anonymous="A2B"`.""",
+        support=SupportLevel.SHOULD,
+        queryable=SupportLevel.MUST,
+        regex=CHEMICAL_FORMULA_REGEXP,
     )
 
-    dimension_types: Tuple[Periodicity, Periodicity, Periodicity] = Field(
+    dimension_types: conlist(Periodicity, min_items=3, max_items=3) = OptimadeField(
         ...,
         description="""List of three integers.
 For each of the three directions indicated by the three lattice vectors (see property `lattice_vectors`), this list indicates if the direction is periodic (value `1`) or non-periodic (value `0`).
@@ -393,10 +444,12 @@ Note: the elements in this list each refer to the direction of the corresponding
     - For a wire along the direction specified by the third lattice vector: `[0, 0, 1]`
     - For a 2D surface/slab, periodic on the plane defined by the first and third lattice vectors: `[1, 0, 1]`
     - For a bulk 3D system: `[1, 1, 1]`""",
+        support=SupportLevel.SHOULD,
+        queryable=SupportLevel.OPTIONAL,
     )
 
-    nperiodic_dimensions: Optional[int] = Field(
-        None,
+    nperiodic_dimensions: int = OptimadeField(
+        ...,
         description="""An integer specifying the number of periodic dimensions in the structure, equivalent to the number of non-zero entries in `dimension_types`.
 
 - **Type**: integer
@@ -413,12 +466,14 @@ Note: the elements in this list each refer to the direction of the corresponding
 - **Query examples**:
     - Match only structures with exactly 3 periodic dimensions: `nperiodic_dimensions=3`
     - Match all structures with 2 or fewer periodic dimensions: `nperiodic_dimensions<=2`""",
+        support=SupportLevel.SHOULD,
+        queryable=SupportLevel.MUST,
     )
 
-    lattice_vectors: Optional[
-        Tuple[Vector3D_unknown, Vector3D_unknown, Vector3D_unknown]
-    ] = Field(
-        None,
+    lattice_vectors: conlist(
+        Vector3D_unknown, min_items=3, max_items=3
+    ) = OptimadeField(
+        ...,
         description="""The three lattice vectors in Cartesian coordinates, in ångström (Å).
 
 - **Type**: list of list of floats or unknown values.
@@ -439,9 +494,11 @@ Note: the elements in this list each refer to the direction of the corresponding
 - **Examples**:
     - `[[4.0,0.0,0.0],[0.0,4.0,0.0],[0.0,1.0,4.0]]` represents a cell, where the first vector is `(4, 0, 0)`, i.e., a vector aligned along the `x` axis of length 4 Å; the second vector is `(0, 4, 0)`; and the third vector is `(0, 1, 4)`.""",
         unit="Å",
+        support=SupportLevel.SHOULD,
+        queryable=SupportLevel.OPTIONAL,
     )
 
-    cartesian_site_positions: List[Vector3D] = Field(
+    cartesian_site_positions: List[Vector3D] = OptimadeField(
         ...,
         description="""Cartesian positions of each site in the structure.
 A site is usually used to describe positions of atoms; what atoms can be encountered at a given site is conveyed by the `species_at_sites` property, and the species themselves are described in the `species` property.
@@ -458,9 +515,11 @@ A site is usually used to describe positions of atoms; what atoms can be encount
 - **Examples**:
     - `[[0,0,0],[0,0,2]]` indicates a structure with two sites, one sitting at the origin and one along the (positive) *z*-axis, 2 Å away from the origin.""",
         unit="Å",
+        support=SupportLevel.SHOULD,
+        queryable=SupportLevel.OPTIONAL,
     )
 
-    nsites: int = Field(
+    nsites: int = OptimadeField(
         ...,
         description="""An integer specifying the length of the `cartesian_site_positions` property.
 
@@ -476,9 +535,11 @@ A site is usually used to describe positions of atoms; what atoms can be encount
 - **Query examples**:
     - Match only structures with exactly 4 sites: `nsites=4`
     - Match structures that have between 2 and 7 sites: `nsites>=2 AND nsites<=7`""",
+        queryable=SupportLevel.MUST,
+        support=SupportLevel.SHOULD,
     )
 
-    species: List[Species] = Field(
+    species: List[Species] = OptimadeField(
         ...,
         description="""A list describing the species of the sites of this structure.
 Species can represent pure chemical elements, virtual-crystal atoms representing a statistical occupation of a given site by multiple chemical elements, and/or a location to which there are attached atoms, i.e., atoms whose precise location are unknown beyond that they are attached to that position (frequently used to indicate hydrogen atoms attached to another element, e.g., a carbon with three attached hydrogens might represent a methyl group, -CH3).
@@ -540,9 +601,11 @@ Species can represent pure chemical elements, virtual-crystal atoms representing
     - `[ {"name": "C12", "chemical_symbols": ["C"], "concentration": [1.0], "mass": 12.0} ]`: any site with this species is occupied by a carbon isotope with mass 12.
     - `[ {"name": "C13", "chemical_symbols": ["C"], "concentration": [1.0], "mass": 13.0} ]`: any site with this species is occupied by a carbon isotope with mass 13.
     - `[ {"name": "CH3", "chemical_symbols": ["C"], "concentration": [1.0], "attached": ["H"], "nattached": [3]} ]`: any site with this species is occupied by a methyl group, -CH3, which is represented without specifying precise positions of the hydrogen atoms.""",
+        support=SupportLevel.SHOULD,
+        queryable=SupportLevel.OPTIONAL,
     )
 
-    species_at_sites: List[str] = Field(
+    species_at_sites: List[str] = OptimadeField(
         ...,
         description="""Name of the species at each site (where values for sites are specified with the same order of the property `cartesian_site_positions`).
 The properties of the species are found in the property `species`.
@@ -562,9 +625,11 @@ The properties of the species are found in the property `species`.
 - **Examples**:
     - `["Ti","O2"]` indicates that the first site is hosting a species labeled `"Ti"` and the second a species labeled `"O2"`.
     - `["Ac", "Ac", "Ag", "Ir"]` indicating the first two sites contains the `"Ac"` species, while the third and fourth sites contain the `"Ag"` and `"Ir"` species, respectively.""",
+        support=SupportLevel.SHOULD,
+        queryable=SupportLevel.OPTIONAL,
     )
 
-    assemblies: Optional[List[Assembly]] = Field(
+    assemblies: Optional[List[Assembly]] = OptimadeField(
         None,
         description="""A description of groups of sites that are statistically correlated.
 
@@ -668,9 +733,11 @@ The properties of the species are found in the property `species`.
         Site 0 is present with a probability of 20 % and site 1 with a probability of 80 %. These two sites are correlated (either site 0 or 1 is present). Similarly, site 2 is present with a probability of 30 % and site 3 with a probability of 70 %.
         These two sites are correlated (either site 2 or 3 is present).
         However, the presence or absence of sites 0 and 1 is not correlated with the presence or absence of sites 2 and 3 (in the specific example, the pair of sites (0, 2) can occur with 0.2*0.3 = 6 % probability; the pair (0, 3) with 0.2*0.7 = 14 % probability; the pair (1, 2) with 0.8*0.3 = 24 % probability; and the pair (1, 3) with 0.8*0.7 = 56 % probability).""",
+        support=SupportLevel.OPTIONAL,
+        queryable=SupportLevel.OPTIONAL,
     )
 
-    structure_features: List[StructureFeatures] = Field(
+    structure_features: List[StructureFeatures] = OptimadeField(
         ...,
         description="""A list of strings that flag which special features are used by the structure.
 
@@ -693,8 +760,66 @@ The properties of the species are found in the property `species`.
         - `assemblies`: this flag MUST be present if the property `assemblies` is present.
 
 - **Examples**: A structure having implicit atoms and using assemblies: `["assemblies", "implicit_atoms"]`""",
+        support=SupportLevel.MUST,
+        queryable=SupportLevel.MUST,
     )
 
+    @validator("chemical_formula_reduced", "chemical_formula_hill")
+    def check_ordered_formula(cls, v, field):
+        if v is None:
+            return v
+
+        numbers = re.findall(r"(\d+\.?)+", v)
+        elements = re.findall(r"[A-Z][a-z]?", v)
+        expected_elements = sorted(elements)
+
+        if field.name == "chemical_formula_hill":
+            for elem in ("C", "H"):
+                if elem in expected_elements:
+                    expected_elements.pop(expected_elements.index(elem))
+                    expected_elements.insert(0, elem)
+
+        if any(elem not in CHEMICAL_SYMBOLS for elem in elements):
+            raise ValueError(
+                f"Cannot use unknown chemical symbols {[elem for elem in elements if elem not in CHEMICAL_SYMBOLS]} in {field.name!r}"
+            )
+
+        if "1" in numbers:
+            raise ValueError(
+                "Must omit proportion '1' from formula {v} in {field.name!r}"
+            )
+        if expected_elements != elements:
+            order = "Hill" if field == "chemical_formula_hill" else "alphabetical"
+            raise ValueError(
+                f"Elements in {field.name!r} must appear in {order} order: {expected_elements} not {elements}."
+            )
+
+        return v
+
+    @validator("chemical_formula_anonymous")
+    def check_anonymous_formula(cls, v):
+        elements = tuple(re.findall(r"[A-Z][a-z]*", v))
+        numbers = [int(n.strip()) for n in re.split(r"[A-Z][a-z]*", v) if n.strip()]
+
+        if any(n == 1 for n in numbers):
+            raise ValueError(
+                "'chemical_formula_anonymous' {v} must omit proportion '1'"
+            )
+
+        expected_labels = ANONYMOUS_ELEMENTS[: len(elements)]
+        expected_numbers = sorted(numbers, reverse=True)
+
+        if expected_numbers != numbers:
+            raise ValueError(
+                f"'chemical_formula_anonymous' {v} has wrong order: elements with highest proportion should appear first: {numbers} vs expected {expected_numbers}"
+            )
+        if elements != expected_labels:
+            raise ValueError(
+                f"'chemical_formula_anonymous' {v} has wrong labels: {elements} vs expected {expected_labels}."
+            )
+
+        return v
+
     @validator("elements", each_item=True)
     def element_must_be_chemical_symbol(cls, v):
         if v not in CHEMICAL_SYMBOLS:
@@ -715,12 +840,6 @@ The properties of the species are found in the property `species`.
             )
         return v
 
-    @validator("chemical_formula_reduced", "chemical_formula_hill")
-    def no_spaces_in_reduced(cls, v):
-        if v and " " in v:
-            raise ValueError(f"Spaces are not allowed, you passed: {v}")
-        return v
-
     @validator("nperiodic_dimensions")
     def check_periodic_dimensions(cls, v, values):
         if values.get("dimension_types", []) and v is None:
@@ -769,7 +888,8 @@ The properties of the species are found in the property `species`.
     def validate_nsites(cls, v, values):
         if v != len(values.get("cartesian_site_positions", [])):
             raise ValueError(
-                f"nsites (value: {v}) MUST equal length of cartesian_site_positions (value: {len(values.get('cartesian_site_positions', []))})"
+                f"nsites (value: {v}) MUST equal length of cartesian_site_positions "
+                f"(value: {len(values.get('cartesian_site_positions', []))})"
             )
         return v
 
@@ -781,7 +901,8 @@ The properties of the species are found in the property `species`.
             )
         if len(v) != values.get("nsites", 0):
             raise ValueError(
-                f"Number of species_at_sites (value: {len(v)}) MUST equal number of sites (value: {values.get('nsites', 'Not specified')})"
+                f"Number of species_at_sites (value: {len(v)}) MUST equal number of sites "
+                f"(value: {values.get('nsites', 'Not specified')})"
             )
         all_species_names = {
             getattr(_, "name", None) for _ in values.get("species", [{}])
@@ -790,7 +911,8 @@ The properties of the species are found in the property `species`.
         for value in v:
             if value not in all_species_names:
                 raise ValueError(
-                    f"species_at_sites MUST be represented by a species' name, but {value} was not found in the list of species names: {all_species_names}"
+                    "species_at_sites MUST be represented by a species' name, "
+                    f"but {value} was not found in the list of species names: {all_species_names}"
                 )
         return v
 
@@ -815,13 +937,15 @@ The properties of the species are found in the property `species`.
             if len(species.chemical_symbols) > 1:
                 if StructureFeatures.DISORDER not in v:
                     raise ValueError(
-                        f"{StructureFeatures.DISORDER.value} MUST be present when any one entry in species has a chemical_symbols list greater than one element"
+                        f"{StructureFeatures.DISORDER.value} MUST be present when any one entry in species "
+                        "has a chemical_symbols list greater than one element"
                     )
                 break
         else:
             if StructureFeatures.DISORDER in v:
                 raise ValueError(
-                    f"{StructureFeatures.DISORDER.value} MUST NOT be present, since all species' chemical_symbols lists are equal to or less than one element"
+                    f"{StructureFeatures.DISORDER.value} MUST NOT be present, since all species' chemical_symbols "
+                    "lists are equal to or less than one element"
                 )
         # assemblies
         if values.get("assemblies", None) is not None:
@@ -832,7 +956,8 @@ The properties of the species are found in the property `species`.
         else:
             if StructureFeatures.ASSEMBLIES in v:
                 raise ValueError(
-                    f"{StructureFeatures.ASSEMBLIES.value} MUST NOT be present, since the property of the same name is not present"
+                    f"{StructureFeatures.ASSEMBLIES.value} MUST NOT be present, "
+                    "since the property of the same name is not present"
                 )
         # site_attachments
         for species in values.get("species", []):
@@ -841,13 +966,15 @@ The properties of the species are found in the property `species`.
             if getattr(species, "attached", None) is not None:
                 if StructureFeatures.SITE_ATTACHMENTS not in v:
                     raise ValueError(
-                        f"{StructureFeatures.SITE_ATTACHMENTS.value} MUST be present when any one entry in species includes attached and nattached"
+                        f"{StructureFeatures.SITE_ATTACHMENTS.value} MUST be present when any one entry "
+                        "in species includes attached and nattached"
                     )
                 break
         else:
             if StructureFeatures.SITE_ATTACHMENTS in v:
                 raise ValueError(
-                    f"{StructureFeatures.SITE_ATTACHMENTS.value} MUST NOT be present, since no species includes the attached and nattached fields"
+                    f"{StructureFeatures.SITE_ATTACHMENTS.value} MUST NOT be present, since no species includes "
+                    "the attached and nattached fields"
                 )
         # implicit_atoms
         species_names = [_.name for _ in values.get("species", [])]
@@ -855,13 +982,15 @@ The properties of the species are found in the property `species`.
             if name not in values.get("species_at_sites", []):
                 if StructureFeatures.IMPLICIT_ATOMS not in v:
                     raise ValueError(
-                        f"{StructureFeatures.IMPLICIT_ATOMS.value} MUST be present when any one entry in species is not represented in species_at_sites"
+                        f"{StructureFeatures.IMPLICIT_ATOMS.value} MUST be present when any one entry in species "
+                        "is not represented in species_at_sites"
                     )
                 break
         else:
             if StructureFeatures.IMPLICIT_ATOMS in v:
                 raise ValueError(
-                    f"{StructureFeatures.IMPLICIT_ATOMS.value} MUST NOT be present, since all species are represented in species_at_sites"
+                    f"{StructureFeatures.IMPLICIT_ATOMS.value} MUST NOT be present, since all species are "
+                    "represented in species_at_sites"
                 )
         return v
 
@@ -869,9 +998,9 @@ The properties of the species are found in the property `species`.
 class StructureResource(EntryResource):
     """Representing a structure."""
 
-    type: str = Field(
+    type: str = StrictField(
         "structures",
-        const=True,
+        const="structures",
         description="""The name of the type of an entry.
 
 - **Type**: string.
@@ -885,6 +1014,9 @@ class StructureResource(EntryResource):
 
 - **Examples**:
     - `"structures"`""",
+        pattern="^structures$",
+        support=SupportLevel.MUST,
+        queryable=SupportLevel.MUST,
     )
 
     attributes: StructureResourceAttributes
