@@ -8,11 +8,7 @@ To implement your own server see the documentation at https://optimade.org/optim
 
 import warnings
 
-from lark.exceptions import VisitError
-
-from pydantic import ValidationError
 from fastapi import FastAPI
-from fastapi.exceptions import RequestValidationError, StarletteHTTPException
 from fastapi.middleware.cors import CORSMiddleware
 
 with warnings.catch_warnings(record=True) as w:
@@ -21,15 +17,11 @@ with warnings.catch_warnings(record=True) as w:
     config_warnings = w
 
 from optimade import __api_version__, __version__
-import optimade.server.exception_handlers as exc_handlers
 from optimade.server.entry_collections import MongoCollection
 from optimade.server.logger import LOGGER
-from optimade.server.middleware import (
-    AddWarnings,
-    CheckWronglyVersionedBaseUrls,
-    EnsureQueryParamIntegrity,
-    HandleApiHint,
-)
+from optimade.server.exception_handlers import OPTIMADE_EXCEPTIONS
+from optimade.server.middleware import OPTIMADE_MIDDLEWARE
+
 from optimade.server.routers import (
     info,
     landing,
@@ -38,7 +30,7 @@ from optimade.server.routers import (
     structures,
     versions,
 )
-from optimade.server.routers.utils import get_providers, BASE_URL_PREFIXES
+from optimade.server.routers.utils import BASE_URL_PREFIXES
 
 
 if CONFIG.config_file is None:
@@ -68,43 +60,41 @@ This specification is generated using [`optimade-python-tools`](https://github.c
 
 if not CONFIG.use_real_mongo:
     import bson.json_util
+    from bson.objectid import ObjectId
     import optimade.server.data as data
-    from .routers import ENTRY_COLLECTIONS
+    from optimade.server.routers import ENTRY_COLLECTIONS
+    from optimade.server.routers.utils import get_providers
 
     def load_entries(endpoint_name: str, endpoint_collection: MongoCollection):
-        LOGGER.debug(f"Loading test {endpoint_name}...")
+        LOGGER.debug("Loading test %s...", endpoint_name)
 
         endpoint_collection.collection.insert_many(getattr(data, endpoint_name, []))
         if endpoint_name == "links":
             LOGGER.debug(
-                "Adding Materials-Consortia providers to links from optimade.org"
+                "  Adding Materials-Consortia providers to links from optimade.org"
             )
-            endpoint_collection.collection.insert_many(
-                bson.json_util.loads(bson.json_util.dumps(get_providers()))
-            )
-        LOGGER.debug(f"Done inserting test {endpoint_name}...")
+            providers = get_providers()
+            for doc in providers:
+                endpoint_collection.collection.replace_one(
+                    filter={"_id": ObjectId(doc["_id"]["$oid"])},
+                    replacement=bson.json_util.loads(bson.json_util.dumps(doc)),
+                    upsert=True,
+                )
+        LOGGER.debug("Done inserting test %s!", endpoint_name)
 
     for name, collection in ENTRY_COLLECTIONS.items():
         load_entries(name, collection)
 
-
-# Add various middleware
+# Add CORS middleware first
 app.add_middleware(CORSMiddleware, allow_origins=["*"])
-app.add_middleware(EnsureQueryParamIntegrity)
-app.add_middleware(CheckWronglyVersionedBaseUrls)
-app.add_middleware(HandleApiHint)
-app.add_middleware(AddWarnings)
 
+# Then add required OPTIMADE middleware
+for middleware in OPTIMADE_MIDDLEWARE:
+    app.add_middleware(middleware)
 
-# Add various exception handlers
-app.add_exception_handler(StarletteHTTPException, exc_handlers.http_exception_handler)
-app.add_exception_handler(
-    RequestValidationError, exc_handlers.request_validation_exception_handler
-)
-app.add_exception_handler(ValidationError, exc_handlers.validation_exception_handler)
-app.add_exception_handler(VisitError, exc_handlers.grammar_not_implemented_handler)
-app.add_exception_handler(NotImplementedError, exc_handlers.not_implemented_handler)
-app.add_exception_handler(Exception, exc_handlers.general_exception_handler)
+# Add exception handlers
+for exception, handler in OPTIMADE_EXCEPTIONS:
+    app.add_exception_handler(exception, handler)
 
 # Add various endpoints to unversioned URL
 for endpoint in (info, links, references, structures, landing, versions):
