@@ -8,11 +8,7 @@ To implement your own index meta-database server see the documentation at https:
 import json
 import warnings
 
-from lark.exceptions import VisitError
-
-from pydantic import ValidationError
 from fastapi import FastAPI
-from fastapi.exceptions import RequestValidationError, StarletteHTTPException
 from fastapi.middleware.cors import CORSMiddleware
 
 with warnings.catch_warnings(record=True) as w:
@@ -21,14 +17,9 @@ with warnings.catch_warnings(record=True) as w:
     config_warnings = w
 
 from optimade import __api_version__, __version__
-import optimade.server.exception_handlers as exc_handlers
 from optimade.server.logger import LOGGER
-from optimade.server.middleware import (
-    AddWarnings,
-    CheckWronglyVersionedBaseUrls,
-    EnsureQueryParamIntegrity,
-    HandleApiHint,
-)
+from optimade.server.exception_handlers import OPTIMADE_EXCEPTIONS
+from optimade.server.middleware import OPTIMADE_MIDDLEWARE
 from optimade.server.routers import index_info, links, versions
 from optimade.server.routers.utils import BASE_URL_PREFIXES
 
@@ -61,43 +52,47 @@ This specification is generated using [`optimade-python-tools`](https://github.c
 
 if not CONFIG.use_real_mongo and CONFIG.index_links_path.exists():
     import bson.json_util
-    from .routers.links import links_coll
-    from .routers.utils import mongo_id_for_database
+    from bson.objectid import ObjectId
+    from optimade.server.routers.links import links_coll
+    from optimade.server.routers.utils import mongo_id_for_database, get_providers
 
     LOGGER.debug("Loading index links...")
     with open(CONFIG.index_links_path) as f:
         data = json.load(f)
 
-        processed = []
+    processed = []
+    for db in data:
+        db["_id"] = {"$oid": mongo_id_for_database(db["id"], db["type"])}
+        processed.append(db)
 
-        for db in data:
-            db["_id"] = {"$oid": mongo_id_for_database(db["id"], db["type"])}
-            processed.append(db)
+    LOGGER.debug(
+        "  Inserting index links into collection from %s...", CONFIG.index_links_path
+    )
+    links_coll.collection.insert_many(
+        bson.json_util.loads(bson.json_util.dumps(processed))
+    )
 
-        LOGGER.debug("Inserting index links into collection...")
-        links_coll.collection.insert_many(
-            bson.json_util.loads(bson.json_util.dumps(processed))
+    LOGGER.debug("  Adding Materials-Consortia providers to links from optimade.org...")
+    providers = get_providers()
+    for doc in providers:
+        links_coll.collection.replace_one(
+            filter={"_id": ObjectId(doc["_id"]["$oid"])},
+            replacement=bson.json_util.loads(bson.json_util.dumps(doc)),
+            upsert=True,
         )
-        LOGGER.debug("Done inserting index links...")
 
+    LOGGER.debug("Done inserting index links!")
 
-# Add various middleware
+# Add CORS middleware first
 app.add_middleware(CORSMiddleware, allow_origins=["*"])
-app.add_middleware(EnsureQueryParamIntegrity)
-app.add_middleware(CheckWronglyVersionedBaseUrls)
-app.add_middleware(HandleApiHint)
-app.add_middleware(AddWarnings)
 
+# Then add required OPTIMADE middleware
+for middleware in OPTIMADE_MIDDLEWARE:
+    app.add_middleware(middleware)
 
-# Add various exception handlers
-app.add_exception_handler(StarletteHTTPException, exc_handlers.http_exception_handler)
-app.add_exception_handler(
-    RequestValidationError, exc_handlers.request_validation_exception_handler
-)
-app.add_exception_handler(ValidationError, exc_handlers.validation_exception_handler)
-app.add_exception_handler(VisitError, exc_handlers.grammar_not_implemented_handler)
-app.add_exception_handler(Exception, exc_handlers.general_exception_handler)
-
+# Add exception handlers
+for exception, handler in OPTIMADE_EXCEPTIONS:
+    app.add_exception_handler(exception, handler)
 
 # Add all endpoints to unversioned URL
 for endpoint in (index_info, links, versions):
