@@ -45,6 +45,12 @@ def validate():
         default=0,
         help="""Increase the verbosity of the output.""",
     )
+    parser.add_argument(
+        "-j",
+        "--json",
+        action="store_true",
+        help="""Only a JSON summary of the validator results will be printed to stdout.""",
+    )
     parser.add_argument(
         "-t",
         "--as-type",
@@ -97,6 +103,7 @@ def validate():
     validator = ImplementationValidator(
         base_url=args["base_url"],
         verbosity=args["verbosity"],
+        respond_json=args["json"],
         as_type=args["as_type"],
         index=args["index"],
         run_optional_tests=not args["skip_optional"],
@@ -104,7 +111,7 @@ def validate():
     )
 
     try:
-        validator.main()
+        validator.validate_implementation()
     # catch and print internal exceptions, exiting with non-zero error code
     except Exception:
         traceback.print_exc()
